@@ -25,7 +25,6 @@
 #endif
 
 #include <panel-applet.h>
-#include <glade/glade.h>
 #include <cairo.h>
 #include <string.h>
 #include <libxml/parser.h>
@@ -38,7 +37,10 @@
 #include <libslab/slab.h>
 
 #include "hard-drive-status-tile.h"
+
+#ifdef HAVE_NETWORK
 #include "network-status-tile.h"
+#endif
 
 #include "tile-table.h"
 
@@ -76,8 +78,8 @@ typedef struct {
 	PanelApplet *panel_applet;
 	GtkWidget   *panel_about_dialog;
 
-	GladeXML *main_menu_xml;
-	GladeXML *panel_button_xml;
+	GtkBuilder *main_menu_ui;
+	GtkBuilder *panel_button_ui;
 
 	GtkToggleButton *panel_buttons [4];
 	GtkToggleButton *panel_button;
@@ -259,32 +261,18 @@ enum {
 
 static Atom slab_action_main_menu_atom = None;
 
-MainMenuUI *
-main_menu_ui_new (PanelApplet *applet)
+static gboolean
+main_menu_delayed_setup (MainMenuUI *this)
 {
-	MainMenuUI        *this;
-	MainMenuUIPrivate *priv;
+	MainMenuUIPrivate *priv = PRIVATE (this);
 
-	gchar *glade_xml_path;
-
-
-	this = g_object_new (MAIN_MENU_UI_TYPE, NULL);
-	priv = PRIVATE (this);
-
-	priv->panel_applet = applet;
-
-	glade_xml_path = g_build_filename (DATADIR, PACKAGE, "slab-window.glade", NULL);
-
-	priv->main_menu_xml    = glade_xml_new (glade_xml_path, "slab-main-menu-window", NULL);
-	priv->panel_button_xml = glade_xml_new (glade_xml_path, "slab-panel-button-root", NULL);
-	g_free (glade_xml_path);
+	if (priv->recently_used_store_monitor != NULL)
+		return FALSE; /* already setup */
 
 	libslab_checkpoint ("main_menu_ui_new(): setup_recently_used_store_monitor");
 	setup_recently_used_store_monitor (this, TRUE);
 	libslab_checkpoint ("main_menu_ui_new(): setup_bookmark_agents");
 	setup_bookmark_agents    (this);
-	libslab_checkpoint ("main_menu_ui_new(): create_panel_button");
-	create_panel_button      (this);
 	libslab_checkpoint ("main_menu_ui_new(): create_slab_window");
 	create_slab_window       (this);
 	libslab_checkpoint ("main_menu_ui_new(): create_search_section");
@@ -319,6 +307,37 @@ main_menu_ui_new (PanelApplet *applet)
 	libslab_checkpoint ("main_menu_ui_new(): apply_lockdown_settings");
 	apply_lockdown_settings (this);
 
+	return FALSE;
+}
+
+MainMenuUI *
+main_menu_ui_new (PanelApplet *applet)
+{
+	MainMenuUI        *this;
+	MainMenuUIPrivate *priv;
+
+	gchar *window_ui_path, *button_ui_path;
+
+
+	this = g_object_new (MAIN_MENU_UI_TYPE, NULL);
+	priv = PRIVATE (this);
+
+	priv->panel_applet = applet;
+
+	window_ui_path = g_build_filename (DATADIR, PACKAGE, "slab-window.ui", NULL);
+	button_ui_path = g_build_filename (DATADIR, PACKAGE, "slab-button.ui", NULL);
+
+	priv->main_menu_ui    = gtk_builder_new ();
+	gtk_builder_add_from_file (priv->main_menu_ui, window_ui_path, NULL);
+	priv->panel_button_ui = gtk_builder_new ();
+	gtk_builder_add_from_file (priv->panel_button_ui, button_ui_path, NULL);
+	g_free (window_ui_path);
+	g_free (button_ui_path);
+
+	libslab_checkpoint ("main_menu_ui_new(): create_panel_button");
+	create_panel_button (this);
+	g_timeout_add_seconds (5, (GSourceFunc) main_menu_delayed_setup, this);
+
 	return this;
 }
 
@@ -340,8 +359,8 @@ main_menu_ui_init (MainMenuUI *this)
 	priv->panel_applet                               = NULL;
 	priv->panel_about_dialog                         = NULL;
 
-	priv->main_menu_xml                              = NULL;
-	priv->panel_button_xml                           = NULL;
+	priv->main_menu_ui                               = NULL;
+	priv->panel_button_ui                            = NULL;
 
 	priv->panel_buttons [PANEL_BUTTON_ORIENT_TOP]    = NULL;
 	priv->panel_buttons [PANEL_BUTTON_ORIENT_BOTTOM] = NULL;
@@ -479,21 +498,24 @@ create_panel_button (MainMenuUI *this)
 	gint i;
 
 
-	button_root = glade_xml_get_widget (
-		priv->panel_button_xml, "slab-panel-button-root");
+	button_root = GTK_WIDGET (gtk_builder_get_object (
+		priv->panel_button_ui, "slab-panel-button-root"));
 
 	gtk_widget_hide (button_root);
 
-	priv->panel_buttons [PANEL_BUTTON_ORIENT_TOP] = GTK_TOGGLE_BUTTON (glade_xml_get_widget (
-		priv->panel_button_xml, "slab-main-menu-panel-button-top"));
-	priv->panel_buttons [PANEL_BUTTON_ORIENT_BOTTOM] = GTK_TOGGLE_BUTTON (glade_xml_get_widget (
-		priv->panel_button_xml, "slab-main-menu-panel-button-bottom"));
-	priv->panel_buttons [PANEL_BUTTON_ORIENT_LEFT] = GTK_TOGGLE_BUTTON (glade_xml_get_widget (
-		priv->panel_button_xml, "slab-main-menu-panel-button-left"));
-	priv->panel_buttons [PANEL_BUTTON_ORIENT_RIGHT] = GTK_TOGGLE_BUTTON (glade_xml_get_widget (
-		priv->panel_button_xml, "slab-main-menu-panel-button-right"));
+	priv->panel_buttons [PANEL_BUTTON_ORIENT_TOP] = GTK_TOGGLE_BUTTON (gtk_builder_get_object (
+		priv->panel_button_ui, "slab-main-menu-panel-button-top"));
+	priv->panel_buttons [PANEL_BUTTON_ORIENT_BOTTOM] = GTK_TOGGLE_BUTTON (gtk_builder_get_object (
+		priv->panel_button_ui, "slab-main-menu-panel-button-bottom"));
+	priv->panel_buttons [PANEL_BUTTON_ORIENT_LEFT] = GTK_TOGGLE_BUTTON (gtk_builder_get_object (
+		priv->panel_button_ui, "slab-main-menu-panel-button-left"));
+	priv->panel_buttons [PANEL_BUTTON_ORIENT_RIGHT] = GTK_TOGGLE_BUTTON (gtk_builder_get_object (
+		priv->panel_button_ui, "slab-main-menu-panel-button-right"));
 
 	for (i = 0; i < 4; ++i) {
+		gtk_widget_set_name (GTK_WIDGET (priv->panel_buttons [i]),
+				     "slab-main-menu-panel-button");
+
 		g_object_set_data (
 			G_OBJECT (priv->panel_buttons [i]), "double-click-detector",
 			double_click_detector_new ());
@@ -541,6 +563,13 @@ create_panel_button (MainMenuUI *this)
 		G_CALLBACK (panel_applet_change_background_cb), this);
 }
 
+static GtkWidget *
+get_widget (MainMenuUIPrivate *priv, const gchar *name)
+{
+	return GTK_WIDGET (gtk_builder_get_object (priv->main_menu_ui, name));
+}
+
+
 static void
 create_slab_window (MainMenuUI *this)
 {
@@ -549,14 +578,13 @@ create_slab_window (MainMenuUI *this)
 	GdkAtom slab_action_atom;
 
 
-	priv->slab_window = glade_xml_get_widget (
-		priv->main_menu_xml, "slab-main-menu-window");
+	priv->slab_window = get_widget (priv, "slab-main-menu-window");
 	gtk_widget_set_app_paintable (priv->slab_window, TRUE);
 	gtk_widget_hide              (priv->slab_window);
 	gtk_window_stick             (GTK_WINDOW (priv->slab_window));
 
-	priv->top_pane  = glade_xml_get_widget (priv->main_menu_xml, "top-pane");
-	priv->left_pane = glade_xml_get_widget (priv->main_menu_xml, "left-pane");
+	priv->top_pane  = get_widget (priv, "top-pane");
+	priv->left_pane = get_widget (priv, "left-pane");
 
 	tomboy_keybinder_init ();
 	tomboy_keybinder_bind ("<Ctrl>Escape", slab_window_tomboy_bindkey_cb, this);
@@ -601,8 +629,8 @@ create_search_section (MainMenuUI *this)
 {
 	MainMenuUIPrivate *priv = PRIVATE (this);
 
-        priv->search_section = glade_xml_get_widget (priv->main_menu_xml, "search-section");
-	priv->search_entry   = glade_xml_get_widget (priv->main_menu_xml, "search-entry");
+	priv->search_section = get_widget (priv, "search-section");
+	priv->search_entry   = get_widget (priv, "search-entry");
 
 	g_signal_connect (
 		G_OBJECT (priv->search_entry), "activate",
@@ -624,19 +652,19 @@ create_file_section (MainMenuUI *this)
 	gint i;
 
 
-	priv->file_section = GTK_NOTEBOOK (glade_xml_get_widget (
-		priv->main_menu_xml, "file-area-notebook"));
+	priv->file_section = GTK_NOTEBOOK (gtk_builder_get_object (
+		priv->main_menu_ui, "file-area-notebook"));
 
-	priv->page_selectors [APPS_PAGE] = glade_xml_get_widget (
-		priv->main_menu_xml, "slab-page-selector-button-applications");
-	priv->page_selectors [DOCS_PAGE] = glade_xml_get_widget (
-		priv->main_menu_xml, "slab-page-selector-button-documents");
-	priv->page_selectors [DIRS_PAGE] = glade_xml_get_widget (
-		priv->main_menu_xml, "slab-page-selector-button-places");
+	priv->page_selectors [APPS_PAGE] =
+        get_widget (priv, "slab-page-selector-button-applications");
+	priv->page_selectors [DOCS_PAGE] =
+        get_widget (priv, "slab-page-selector-button-documents");
+	priv->page_selectors [DIRS_PAGE] =
+        get_widget (priv, "slab-page-selector-button-places");
 
-	pages [APPS_PAGE] = glade_xml_get_widget (priv->main_menu_xml, "applications-page");
-	pages [DOCS_PAGE] = glade_xml_get_widget (priv->main_menu_xml, "documents-page");
-	pages [DIRS_PAGE] = glade_xml_get_widget (priv->main_menu_xml, "places-page");
+	pages [APPS_PAGE] = get_widget (priv, "applications-page");
+	pages [DOCS_PAGE] = get_widget (priv, "documents-page");
+	pages [DIRS_PAGE] = get_widget (priv, "places-page");
 
 	for (i = 0; i < 3; ++i) {
 		gtk_container_child_get (
@@ -654,16 +682,16 @@ create_file_section (MainMenuUI *this)
 	priv->current_page_gconf_mntr_id = libslab_gconf_notify_add (
 		CURRENT_PAGE_GCONF_KEY, current_page_notify_cb, this);
 
-	priv->table_sections [USER_APPS_TABLE] = glade_xml_get_widget (
-		priv->main_menu_xml, "user-apps-section");
-	priv->table_sections [RCNT_APPS_TABLE] = glade_xml_get_widget (
-		priv->main_menu_xml, "recent-apps-section");
-	priv->table_sections [USER_DOCS_TABLE] = glade_xml_get_widget (
-		priv->main_menu_xml, "user-docs-section");
-	priv->table_sections [RCNT_DOCS_TABLE] = glade_xml_get_widget (
-		priv->main_menu_xml, "recent-docs-section");
-	priv->table_sections [USER_DIRS_TABLE] = glade_xml_get_widget (
-		priv->main_menu_xml, "user-dirs-section");
+	priv->table_sections [USER_APPS_TABLE] =
+        get_widget (priv, "user-apps-section");
+	priv->table_sections [RCNT_APPS_TABLE] =
+        get_widget (priv, "recent-apps-section");
+	priv->table_sections [USER_DOCS_TABLE] =
+        get_widget (priv, "user-docs-section");
+	priv->table_sections [RCNT_DOCS_TABLE] =
+        get_widget (priv, "recent-docs-section");
+	priv->table_sections [USER_DIRS_TABLE] =
+        get_widget (priv, "user-dirs-section");
 }
 
 static void
@@ -674,8 +702,8 @@ create_system_section (MainMenuUI *this)
 	GtkContainer *ctnr;
 
 
-	ctnr = GTK_CONTAINER (glade_xml_get_widget (
-		priv->main_menu_xml, "system-item-table-container"));
+	ctnr = GTK_CONTAINER (gtk_builder_get_object (
+		priv->main_menu_ui, "system-item-table-container"));
 
 	priv->sys_table = TILE_TABLE (tile_table_new (
 		priv->bm_agents [BOOKMARK_STORE_SYSTEM], -1, 1, TRUE, TRUE,
@@ -689,8 +717,7 @@ create_system_section (MainMenuUI *this)
 		G_OBJECT (priv->sys_table), "notify::" TILE_TABLE_TILES_PROP,
 		G_CALLBACK (tile_table_notify_cb), this);
 
-	priv->system_section = glade_xml_get_widget (
-		priv->main_menu_xml, "slab-system-section");
+	priv->system_section = get_widget (priv, "slab-system-section");
 }
 
 static void
@@ -704,8 +731,8 @@ create_status_section (MainMenuUI *this)
 	gint icon_width;
 
 
-	ctnr = GTK_CONTAINER (glade_xml_get_widget (
-		priv->main_menu_xml, "hard-drive-status-container"));
+	ctnr = GTK_CONTAINER (gtk_builder_get_object (
+		priv->main_menu_ui, "hard-drive-status-container"));
 	tile = hard_drive_status_tile_new ();
 
 	gtk_icon_size_lookup (GTK_ICON_SIZE_DND, & icon_width, NULL);
@@ -716,8 +743,9 @@ create_status_section (MainMenuUI *this)
 	gtk_container_add   (ctnr, tile);
 	gtk_widget_show_all (GTK_WIDGET (ctnr));
 
-	ctnr = GTK_CONTAINER (glade_xml_get_widget (
-		priv->main_menu_xml, "network-status-container"));
+#ifdef HAVE_NETWORK
+	ctnr = GTK_CONTAINER (gtk_builder_get_object (
+		priv->main_menu_ui, "network-status-container"));
 	priv->network_status = network_status_tile_new ();
 
 	gtk_widget_set_size_request (priv->network_status, 6 * icon_width, -1);
@@ -728,9 +756,10 @@ create_status_section (MainMenuUI *this)
 
 	gtk_container_add   (ctnr, priv->network_status);
 	gtk_widget_show_all (GTK_WIDGET (ctnr));
+#endif
+
+	priv->status_section = get_widget (priv, "slab-status-section");
 
-	priv->status_section = glade_xml_get_widget (
-		priv->main_menu_xml, "slab-status-section");
 }
 
 static void
@@ -741,8 +770,8 @@ create_user_apps_section (MainMenuUI *this)
 	GtkContainer *ctnr;
 
 
-	ctnr = GTK_CONTAINER (glade_xml_get_widget (
-		priv->main_menu_xml, "user-apps-table-container"));
+	ctnr = GTK_CONTAINER (gtk_builder_get_object (
+		priv->main_menu_ui, "user-apps-table-container"));
 
 	priv->file_tables [USER_APPS_TABLE] = TILE_TABLE (tile_table_new (
 		priv->bm_agents [BOOKMARK_STORE_USER_APPS], -1, 2, TRUE, TRUE,
@@ -759,8 +788,8 @@ create_rct_apps_section (MainMenuUI *this)
 	GtkContainer *ctnr;
 
 
-	ctnr = GTK_CONTAINER (glade_xml_get_widget (
-		priv->main_menu_xml, "recent-apps-table-container"));
+	ctnr = GTK_CONTAINER (gtk_builder_get_object (
+		priv->main_menu_ui, "recent-apps-table-container"));
 
 	priv->file_tables [RCNT_APPS_TABLE] = TILE_TABLE (tile_table_new (
 		priv->bm_agents [BOOKMARK_STORE_RECENT_APPS], -1, 2, FALSE, FALSE,
@@ -777,8 +806,8 @@ create_user_docs_section (MainMenuUI *this)
 	GtkContainer *ctnr;
 
 
-	ctnr = GTK_CONTAINER (glade_xml_get_widget (
-		priv->main_menu_xml, "user-docs-table-container"));
+	ctnr = GTK_CONTAINER (gtk_builder_get_object (
+		priv->main_menu_ui, "user-docs-table-container"));
 
 	priv->file_tables [USER_DOCS_TABLE] = TILE_TABLE (tile_table_new (
 		priv->bm_agents [BOOKMARK_STORE_USER_DOCS], -1, 2, TRUE, TRUE,
@@ -795,8 +824,8 @@ create_rct_docs_section (MainMenuUI *this)
 	GtkContainer *ctnr;
 
 
-	ctnr = GTK_CONTAINER (glade_xml_get_widget (
-		priv->main_menu_xml, "recent-docs-table-container"));
+	ctnr = GTK_CONTAINER (gtk_builder_get_object (
+		priv->main_menu_ui, "recent-docs-table-container"));
 
 	priv->file_tables [RCNT_DOCS_TABLE] = TILE_TABLE (tile_table_new (
 		priv->bm_agents [BOOKMARK_STORE_RECENT_DOCS], -1, 2, FALSE, FALSE,
@@ -819,8 +848,8 @@ create_user_dirs_section (MainMenuUI *this)
 	GtkContainer *ctnr;
 
 
-	ctnr = GTK_CONTAINER (glade_xml_get_widget (
-		priv->main_menu_xml, "user-dirs-table-container"));
+	ctnr = GTK_CONTAINER (gtk_builder_get_object (
+		priv->main_menu_ui, "user-dirs-table-container"));
 
 	priv->file_tables [USER_DIRS_TABLE] = TILE_TABLE (tile_table_new (
 		priv->bm_agents [BOOKMARK_STORE_USER_DIRS], -1, 2, FALSE, FALSE,
@@ -837,9 +866,9 @@ create_more_buttons (MainMenuUI *this)
 	gint i;
 
 
-	priv->more_buttons [0] = glade_xml_get_widget (priv->main_menu_xml, "more-applications-button");
-	priv->more_buttons [1] = glade_xml_get_widget (priv->main_menu_xml, "more-documents-button");
-	priv->more_buttons [2] = glade_xml_get_widget (priv->main_menu_xml, "more-places-button");
+	priv->more_buttons [0] = get_widget (priv, "more-applications-button");
+	priv->more_buttons [1] = get_widget (priv, "more-documents-button");
+	priv->more_buttons [2] = get_widget (priv, "more-places-button");
 
 	for (i = 0; i < 3; ++i) {
 		g_object_set_data (
@@ -851,9 +880,9 @@ create_more_buttons (MainMenuUI *this)
 			G_CALLBACK (more_buttons_clicked_cb), this);
 	}
 
-	priv->more_sections [0] = glade_xml_get_widget (priv->main_menu_xml, "more-apps-section");
-	priv->more_sections [1] = glade_xml_get_widget (priv->main_menu_xml, "more-docs-section");
-	priv->more_sections [2] = glade_xml_get_widget (priv->main_menu_xml, "more-dirs-section");
+	priv->more_sections [0] = get_widget (priv, "more-apps-section");
+	priv->more_sections [1] = get_widget (priv, "more-docs-section");
+	priv->more_sections [2] = get_widget (priv, "more-dirs-section");
 }
 
 static void
@@ -1064,7 +1093,7 @@ item_to_user_doc_tile (BookmarkItem *item, gpointer data)
 			return TILE (document_tile_new_force_icon (item->uri, item->mime_type,
 				item->mtime, "gnome-mime-application-vnd.oasis.opendocument.text-template"));
 	}
-	return TILE (document_tile_new (item->uri, item->mime_type, item->mtime));
+	return TILE (document_tile_new (BOOKMARK_STORE_USER_DOCS, item->uri, item->mime_type, item->mtime));
 }
 
 static Tile *
@@ -1111,7 +1140,7 @@ item_to_recent_doc_tile (BookmarkItem *item, gpointer data)
 	if (bookmark_agent_has_item (priv->bm_agents [BOOKMARK_STORE_USER_DOCS], item->uri))
 		return NULL;
 
-	return TILE (document_tile_new (item->uri, item->mime_type, item->mtime));
+	return TILE (document_tile_new (BOOKMARK_STORE_RECENT_DOCS, item->uri, item->mime_type, item->mtime));
 }
 
 static Tile *
@@ -1313,7 +1342,7 @@ update_limits (MainMenuUI *this)
 	priv->max_total_items = max_total_items_default;
 
 	for (i = 0; i < 2; ++i) {
-		if (GTK_WIDGET_VISIBLE (sections [i]))
+		if (gtk_widget_get_visible (sections [i]))
 			g_object_get (
 				user_tables [i],
 				"n-rows", & n_rows, "n-columns", & n_cols, NULL);
@@ -1432,22 +1461,22 @@ set_table_section_visible (MainMenuUI *this, TileTable *table)
 		gtk_widget_hide (priv->table_sections [table_id]);
 
 	if (
-		GTK_WIDGET_VISIBLE (priv->table_sections [USER_APPS_TABLE]) ||
-		GTK_WIDGET_VISIBLE (priv->table_sections [RCNT_APPS_TABLE])
+		gtk_widget_get_visible (priv->table_sections [USER_APPS_TABLE]) ||
+		gtk_widget_get_visible (priv->table_sections [RCNT_APPS_TABLE])
 	)
 		gtk_widget_show (priv->page_selectors [APPS_PAGE]);
 	else
 		gtk_widget_hide (priv->page_selectors [APPS_PAGE]);
 
 	if (
-		GTK_WIDGET_VISIBLE (priv->table_sections [USER_DOCS_TABLE]) ||
-		GTK_WIDGET_VISIBLE (priv->table_sections [RCNT_DOCS_TABLE])
+		gtk_widget_get_visible (priv->table_sections [USER_DOCS_TABLE]) ||
+		gtk_widget_get_visible (priv->table_sections [RCNT_DOCS_TABLE])
 	)
 		gtk_widget_show (priv->page_selectors [DOCS_PAGE]);
 	else
 		gtk_widget_hide (priv->page_selectors [DOCS_PAGE]);
 
-	if (GTK_WIDGET_VISIBLE (priv->table_sections [USER_DIRS_TABLE]))
+	if (gtk_widget_get_visible (priv->table_sections [USER_DIRS_TABLE]))
 		gtk_widget_show (priv->page_selectors [DIRS_PAGE]);
 	else
 		gtk_widget_hide (priv->page_selectors [DIRS_PAGE]);
@@ -1834,7 +1863,9 @@ present_slab_window (MainMenuUI *this)
 {
 	MainMenuUIPrivate *priv = PRIVATE (this);
 
+#ifdef HAVE_NETWORK
 	network_tile_update_status (priv->network_status);
+#endif
 
 	update_recently_used_sections (this);
 
@@ -1854,11 +1885,12 @@ panel_button_clicked_cb (GtkButton *button, gpointer user_data)
 
 	gboolean visible;
 
+	main_menu_delayed_setup (this);
 
 	detector = DOUBLE_CLICK_DETECTOR (
 		g_object_get_data (G_OBJECT (toggle), "double-click-detector"));
 
-	visible = GTK_WIDGET_VISIBLE (priv->slab_window);
+	visible = gtk_widget_get_visible (priv->slab_window);
 
 	if (! double_click_detector_is_double_click (detector, gtk_get_current_event_time (), TRUE)) {
 		if (! visible)
@@ -1866,7 +1898,7 @@ panel_button_clicked_cb (GtkButton *button, gpointer user_data)
 		else
 			gtk_widget_hide (priv->slab_window);
 
-		visible = GTK_WIDGET_VISIBLE (priv->slab_window);
+		visible = gtk_widget_get_visible (priv->slab_window);
 	}
 
 	gtk_toggle_button_set_active (priv->panel_button, visible);
@@ -2155,11 +2187,11 @@ slab_window_allocate_cb (GtkWidget *widget, GtkAllocation *alloc, gpointer user_
 
 	if (orient == PANEL_APPLET_ORIENT_UP || orient == PANEL_APPLET_ORIENT_DOWN) {
 		if ((slab_geom.x + slab_geom.width) > (monitor_geom.x + monitor_geom.width))
-			slab_geom.x = button_geom.x + button_geom.width - slab_geom.width;
+			slab_geom.x = MAX (monitor_geom.x, monitor_geom.x + monitor_geom.width - slab_geom.width);
 	}
 	else {
 		if ((slab_geom.y + slab_geom.height) > (monitor_geom.y + monitor_geom.height))
-			slab_geom.y = button_geom.y + button_geom.height - slab_geom.height;
+			slab_geom.y = MAX (monitor_geom.y, monitor_geom.y + monitor_geom.height - slab_geom.height);
 	}
 
 	gtk_window_move (GTK_WINDOW (priv->slab_window), slab_geom.x, slab_geom.y);
@@ -2311,10 +2343,14 @@ more_buttons_clicked_cb (GtkButton *button, gpointer user_data)
 		if (GTK_WIDGET (button) == priv->more_buttons [APPS_PAGE])
 			ditem_id = libslab_get_gconf_value (APP_BROWSER_GCONF_KEY);
 		else if (GTK_WIDGET (button) == priv->more_buttons [DOCS_PAGE]) {
-			dir = g_build_filename (g_get_home_dir (), "Documents", NULL);
+			dir = g_strdup (g_get_user_special_dir (G_USER_DIRECTORY_DOCUMENTS));
+			if (! dir)
+			      dir = g_build_filename (g_get_home_dir (), "Documents", NULL);
 
-			if (! g_file_test (dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))
+			if (! g_file_test (dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
+				g_free (dir);
 				dir = g_strdup (g_get_home_dir ());
+			}
 
 			uri = g_filename_to_uri (dir, NULL, NULL);
 
@@ -2378,6 +2414,7 @@ panel_menu_about_cb (BonoboUIComponent *component, gpointer user_data, const gch
 	MainMenuUI *this        = MAIN_MENU_UI (user_data);
 	MainMenuUIPrivate *priv = PRIVATE      (this);
 
+	main_menu_delayed_setup (this);
 
 	if (! priv->panel_about_dialog) {
 		priv->panel_about_dialog = gtk_about_dialog_new ();
